#!/bin/bash

  VERSION="1.00"

  BBLUE=$( echo -en "\033[44m")
  BWHITE=$(echo -en "\033[47m")
  BBLACK=$(echo -en "\033[40m")
  FWHITE=$(echo -en "\033[37;1m")
  FBLACK=$(echo -en "\033[30m")
  FRED=$(  echo -en "\033[31;1m")
  FBLUE=$( echo -en "\033[34;1m")
  FGREEN=$(echo -en "\033[32;1m")
  FCYAN=$( echo -en "\033[35;1m")
  NORMAL=$(echo -en "\033[39;0m")

  STITLE="${FWHITE}${BBLUE}"
  SFORM="${FBLACK}${BWHITE}"
  SNUM="${FBLACK}${BWHITE}"
  SKEY="${FGREEN}${BWHITE}"
  SEQU="${FRED}${BWHITE}"
  SCOM="${FBLUE}${BWHITE}"
  SSHADOW="${BBLACK}"

  CURX=0
  CURY=0
  IDX=65

function aboutit()
{
  echo "Omnicalc Font Editor for bash v${VERSION}"
  echo "Usage: $(basename $0) [8xfont-file]"
  echo ""
  echo "This script can edit, save and load Omnicalc font file (*.8xfont)"
  echo "and compile to TI-83/84 Plus format (*.8xp)."
  echo "This script needs commands as follows: dialog, printf"
  echo ""
  echo "Keys:"
  echo "    [hjkl] - Move the cursor (also Allow keys)"
  echo "    Space  - Edit a selected pixel (also Tab, Enter and BackSpace)"
  echo "    b      - Move to a previous character"
  echo "    n      - Move to a next character"
  echo "    1      - Load from 8xfont file"
  echo "    2      - Save as 8xfont file"
  echo "    3      - Compile to 8xp file"
  echo "    q      - Quit from this script"
  echo ""
  echo "Version Info:"
  echo "    v1.00 - First released version (2006-03-04)"
  echo ""
  echo "This script was written by Yuki Mio (miotti@plum.ocn.ne.jp)."
  echo "[http://www.ticalc.org/archives/files/authors/70/7071.html]"
  exit 0
}

function quitit()
{
  stty echo &>/dev/null
  clear
  exit 1
}

function compileit()
{

  clear
  stty echo &>/dev/null
  echo "Input On-Calc-Name:"
  read TMP
  CNAME=$(tr 'a-z' 'A-Z' <<< $TMP)
  FLAG=$(tr -d 'A-Z0-9' <<< $CNAME)

    if [ "$FLAG " != " " ]
    then
      errorby "On-Calc-Name is invalid. Please use [0-9A-Za-z]."
      return 1
    fi

    if((${#CNAME} < 1 || ${#CNAME} > 8))
    then
      errorby "The length of On-Calc-Name is invalid. Please input [1-8] characters."
      return 1
    fi

  grep "^[0-9].*" <<< $CNAME &>/dev/null
    if(($? == 0))
    then
      errorby "On-Calc-Name is invalid. The first character must be an alphabet."
      return 1
    fi

    while((${#CNAME} < 8))
    do
      CNAME="${CNAME}-"
    done

  clear
  stty -echo &>/dev/null
  echo "Opening \"$1\""
  echo -n "" 2>/dev/null > "$1"
    if(($? != 0))
    then
      errorby "Could not open \"$1\". Please check permission of it."
      return 1
    fi
  echo "Compiling... Wait a minute..."

  CHK="1412"

  #File header
  wbin 42 42 84 73 56 51 70 42 26 10 0 >> "$1"

  #Comment
  echo -n "Generated by Omnicalc Font Editor 4 bash" >> "$1"
  wbin 0 >> "$1"

  #Delimiter
  wbin 0 >> "$1"

  #Datasize1
  wbin 30 7 >> "$1"

  #Ramdom1
  wbin 13 0 >> "$1"

  #Datasize2
  wbin 13 7 >> "$1"

  #Protected
  wbin 6 >> "$1"

  #Name
  TMP=$(sed s/'\(.\)'/' \1 '/g <<< $CNAME | \
        sed s/' 0 '/' 48 '/g | \
        sed s/' 1 '/' 49 '/g | \
        sed s/' 2 '/' 50 '/g | \
        sed s/' 3 '/' 51 '/g | \
        sed s/' 4 '/' 52 '/g | \
        sed s/' 5 '/' 53 '/g | \
        sed s/' 6 '/' 54 '/g | \
        sed s/' 7 '/' 55 '/g | \
        sed s/' 8 '/' 56 '/g | \
        sed s/' 9 '/' 57 '/g | \
        sed s/' A '/' 65 '/g | \
        sed s/' B '/' 66 '/g | \
        sed s/' C '/' 67 '/g | \
        sed s/' D '/' 68 '/g | \
        sed s/' E '/' 69 '/g | \
        sed s/' F '/' 70 '/g | \
        sed s/' G '/' 71 '/g | \
        sed s/' H '/' 72 '/g | \
        sed s/' I '/' 73 '/g | \
        sed s/' J '/' 74 '/g | \
        sed s/' K '/' 75 '/g | \
        sed s/' L '/' 76 '/g | \
        sed s/' M '/' 77 '/g | \
        sed s/' N '/' 78 '/g | \
        sed s/' O '/' 79 '/g | \
        sed s/' P '/' 80 '/g | \
        sed s/' Q '/' 81 '/g | \
        sed s/' R '/' 82 '/g | \
        sed s/' S '/' 83 '/g | \
        sed s/' T '/' 84 '/g | \
        sed s/' U '/' 85 '/g | \
        sed s/' V '/' 86 '/g | \
        sed s/' W '/' 87 '/g | \
        sed s/' X '/' 88 '/g | \
        sed s/' Y '/' 89 '/g | \
        sed s/' Z '/' 90 '/g | \
        sed s/' - '/' 0  '/g)
  wbin $TMP >> "$1"
  SUM=$(sumof $TMP)
  ((CHK+=SUM))

  #Ramdom2
  wbin 0 0 >> "$1"

  #Datasize3
  wbin 13 7 >> "$1"

  #Datasize4
  wbin 11 7 >> "$1"

  #Font header
  wbin 187 109 201 111 109 110 105 99 97 108 99 >> "$1"

  #Font data
  TMP=$(sed s/' '//g <<< ${CFONTS[@]} | \
        sed s/'\(.....\)'/'\1 '/g | \
        sed s/00000/0/g | \
        sed s/00001/1/g | \
        sed s/00010/2/g | \
        sed s/00011/3/g | \
        sed s/00100/4/g | \
        sed s/00101/5/g | \
        sed s/00110/6/g | \
        sed s/00111/7/g | \
        sed s/01000/8/g | \
        sed s/01001/9/g | \
        sed s/01010/10/g | \
        sed s/01011/11/g | \
        sed s/01100/12/g | \
        sed s/01101/13/g | \
        sed s/01110/14/g | \
        sed s/01111/15/g | \
        sed s/10000/16/g | \
        sed s/10001/17/g | \
        sed s/10010/18/g | \
        sed s/10011/19/g | \
        sed s/10100/20/g | \
        sed s/10101/21/g | \
        sed s/10110/22/g | \
        sed s/10111/23/g | \
        sed s/11000/24/g | \
        sed s/11001/25/g | \
        sed s/11010/26/g | \
        sed s/11011/27/g | \
        sed s/11100/28/g | \
        sed s/11101/29/g | \
        sed s/11110/30/g | \
        sed s/11111/31/g)
  wbin $TMP >> "$1"
  SUM=$(sumof $TMP)
  ((CHK+=SUM))

  #Checksum
  ((CHK=CHK%65536))
  wbin $((CHK%256)) $((CHK/256)) >> "$1"

}

function saveit()
{
  clear
  echo "Opening \"$1\""
  echo -n "" 2>/dev/null > "$1"
    if(($? != 0))
    then
      errorby "Could not open \"$1\". Please check permission of it."
      return 1
    fi
  echo -n "Saving..."
  sed s/' '//g <<< ${CFONTS[@]} | sed s/'\(0\|1\)'/' \1 \n'/g | sed s/0/255/g | sed s/1/0/g >> "$1"
}

function loadit()
{

  file "$1" &>/dev/null
    if(($? != 0))
    then
      errorby "File \"$1\" is not found."
      return 1
    fi

  clear
  echo "Opening \"$1\""
  echo "Checking file format... Wait a minute..."

  TMP=$(nkf -eLu "$1" | sed s/0/1/g | sed s/255/0/g)
  BUF=$(sed s/' '//g <<< $TMP)
    if((${#BUF} != 8960))
    then
      errorby "File format is invalid. This file is not a font file."
      return 1
    fi

  CHK=$(sed s/'0\|1'//g <<< $BUF)
    if((${#CHK} != 0))
    then
      errorby "File format is invalid. This file is not a font file."
      return 1
    fi

  echo -n "Loading"
    for((i=0; i<256; i++))
    do
      CFONTS[$i]=${BUF:$((i*35)):35}
        ((i%16 == 0)) && echo -n "."
    done

  return 0
}

function redraw()
{
  echo -en "\033[0;0H"

  ((CURPOS=CURY*5+CURX))

    for((i=0; i<7; i++))
    do
      YLINE=${CFONTS[$IDX]:$((i*5)):5}
      BUF="${FRED}"
        for((j=0; j<5; j++))
        do
          C=${YLINE:$j:1}
            ((C == 0)) && BUF="${BUF}${BWHITE}" || BUF="${BUF}${BBLACK}"
            ((i*5+j == CURPOS)) && BUF="${BUF}[]" || BUF="${BUF}  "
        done
      LINES[$i]="${BUF}${NORMAL}"
    done

  CIDX=$(printf "%03d" $IDX)
  TEXTS=(" ${SNUM}<--[${CIDX}]-->   " \
         "${SKEY}Space${SEQU} = ${SCOM}Edit   " \
         "${SKEY}[ b ]${SEQU} = ${SCOM}Back   " \
         "${SKEY}[ n ]${SEQU} = ${SCOM}Next   " \
         "${SKEY}[ 1 ]${SEQU} = ${SCOM}Load   " \
         "${SKEY}[ 2 ]${SEQU} = ${SCOM}Save   " \
         "${SKEY}[ 3 ]${SEQU} = ${SCOM}Compile")
  echo "${STITLE}  Omnicalc Font Editor v${VERSION}  ${NORMAL}"
  echo "${SFORM}                              ${SSHADOW}  ${NORMAL}"
    for((i=0; i<7; i++))
    do
      echo "${SFORM}  ${LINES[$i]}${SFORM}  ${TEXTS[$i]} ${SSHADOW}  ${NORMAL}"
    done
  echo "${SFORM}                              ${SSHADOW}  ${NORMAL}"
  echo -n "  ${SSHADOW}                              ${NORMAL}  "
}

function wbin()
{
    (($# == 0)) && return 1
    for DATA in $@
    do
      echo -en $(printf "\%03o" $DATA)
    done
}

function sumof()
{
  SUMOF=0
    for DATA in $@
    do
        ((SUMOF+=DATA))
    done
  echo $SUMOF
}

function errorby()
{
  echo "$1"
  echo "[Press any key]"
  read -s -n 1 TMP
}

function initit()
{
  CFONTS=(00000000000000000000000000000000000 00000000001011001001010011001010010 \
          00000000001000110001100011001101101 00000000001000110001010100101000100 \
          00000000001000110001101011010101010 00000010000110001110011000100000000 \
          00100011101111101110011100000000000 00000000000111001110111110111000100 \
          00010001010010000100001001010001000 00000100010101000100010101000100000 \
          00000000000000001110010100111000000 00000000000000000100011100010000000 \
          00000000000000000000001000000000000 00000000000000011100010000100001000 \
          01100000100110000010011000000000000 11111110001100011110110001100011000 \
          00111001000010000100101000110000100 00011000011100100001000010000000000 \
          01100000100010001000011100000000000 00000000000000100010001000100011111 \
          01100100101001001100000000000000000 10110110001000010000000000000000000 \
          01110001000010000100001000000000000 00001001101100000110000010000011111 \
          00010000101111100100111110100001000 10000011000001101100100000000011111 \
          00000000000011100000000000000000000 00000000000111101000011100100001111 \
          00000001000001011111000100010000000 00000000001011110101101011010110111 \
          00100011101010100100001000010000100 00100001000010000100101010111000100 \
          00000000000000000000000000000000000 00100001000010000100000000010000100 \
          01010010100101000000000000000000000 01010010101111101010111110101001010 \
          01000010100111000010000100000000000 11000110010001000100010001001100011 \
          01000101001010001000101011001001101 00100001000010000000000000000000000 \
          00010001000100001000010000010000010 01000001000001000010000100010001000 \
          00000001001010101110101010010000000 00000001000010011111001000010000000 \
          00000000000000000000011000010001000 00000000000000011111000000000000000 \
          00000000000000000000000000110001100 00000000010001000100010001000000000 \
          01110100011001110101110011000101110 00100011000010000100001000010001110 \
          01110100010000100010001000100011111 11111000100010000010000011000101110 \
          00010001100101010010111110001000010 11111100001111000001000011000101110 \
          00110010001000011110100011000101110 11111000010001000100010000100001000 \
          01110100011000101110100011000101110 01110100011000101111000010001001100 \
          00000011000110000000011000110000000 00000011000110000000011000010001000 \
          00010001000100010000010000010000010 00000000001111100000111110000000000 \
          01000001000001000001000100010001000 01110100010000100010001000000000100 \
          01110100011010110111101001000001111 01110100011000111111100011000110001 \
          11110100011000111110100011000111110 01110100011000010000100001000101110 \
          11110100011000110001100011000111110 11111100001000011110100001000011111 \
          11111100001000011110100001000010000 01110100011000010111100011000101111 \
          10001100011000111111100011000110001 01110001000010000100001000010001110 \
          00111000100001000010000101001001100 10001100101010011000101001001010001 \
          10000100001000010000100001000011111 10001110111010110101100011000110001 \
          10001100011100110101100111000110001 01110100011000110001100011000101110 \
          11110100011000111110100001000010000 01110100011000110001101011001001101 \
          11110100011000111110101001001010001 01111100001000001110000010000111110 \
          11111001000010000100001000010000100 10001100011000110001100011000101110 \
          10001100011000110001010100101000100 10001100011000110001101011010101010 \
          10001100010101000100010101000110001 10001100011000101010001000010000100 \
          11111000010001000100010001000011111 00110010011000111111100011001001100 \
          00000100000100000100000100000100000 01100001000010000100001000010001100 \
          00100010101000100000000000000000000 00000000000000000000000000000011111 \
          00100001000001000000000000000000000 00000000000111000001011111000101111 \
          10000100001011011001100011000111110 00000000000111010000100001000101110 \
          00001000010110110011100011000101111 00000000000111010001111111000001110 \
          00110010010100011100010000100001000 00000011111000110001011110000101110 \
          10000100001011011001100011000110001 00100000000110000100001000010001110 \
          00010000000011000010000101001001100 01000010000100101010011000101001001 \
          01100001000010000100001000010001110 00000000001101010101101011000110001 \
          00000000001011011001100011000110001 00000000000111010001100011000101110 \
          00000000001111010001111101000010000 00000000000110110011011110000100001 \
          00000000001011011001100001000010000 00000000000111010000011100000111110 \
          01000010001110001000010000100100110 00000000001000110001100011001101101 \
          00000000001000110001100010101000100 00000000001000110001101011010101010 \
          00000000001000101010001000101010001 00000000001000110001011110000101110 \
          00000000001111100010001000100011111 00011001000010001000001000010000011 \
          00100001000010000100001000010000100 11000001000010000010001000010011000 \
          00000010001010100010000000000000000 11111111111000111111100011111111111 \
          00000000000111001010010100101001110 00000000000010001100001000010000100 \
          00000000000110000010001000100001110 00000000000110000010001000001001100 \
          00000000000100001010011100001000010 00000000000111001000011000001001100 \
          00000000000011001000011100101001110 00000000000111000010001000100001000 \
          00000000000111001010011100101001110 00000000000111001010011100001001100 \
          00010001000111010001100011111110001 01000001000111010001100011111110001 \
          00100010100000001110100011111110001 01010000000111010001100011111110001 \
          00010001000111000001011111000101111 01000001000111000001011111000101111 \
          00100010100111000001011111000101111 01010000000111000001011111000101111 \
          00010001001111110000111101000011111 01000001001111110000111101000011111 \
          00100010101111110000111101000011111 01010000001111110000111101000011111 \
          00010001000111010001111111000001110 01000001000111010001111111000001110 \
          00100010100111010001111111000001110 01010000000111010001111111000001110 \
          00010001000111000100001000010001110 01000001000111000100001000010001110 \
          00100010100111000100001000010001110 01010000000111000100001000010001110 \
          00010001000000001100001000010001110 01000001000000001100001000010001110 \
          00100010100000001100001000010001110 01010000000000001100001000010001110 \
          00010001000111010001100011000101110 01000001000111010001100011000101110 \
          00100010100111010001100011000101110 01010000000111010001100011000101110 \
          00010001000000001110100011000101110 01000001000000001110100011000101110 \
          00100010100000001110100011000101110 01010000000000001110100011000101110 \
          00010001001000110001100011000101110 01000001001000110001100011000101110 \
          00100010101000110001100011000101110 01010000001000110001100011000101110 \
          00010001001000110001100011001101101 01000001001000110001100011001101101 \
          00100010100000010001100011001101101 01010000001000110001100011001101101 \
          01110100011000010001011100010011100 00000000000111010000100010111011100 \
          00101010101000111001101011001110001 00101010100000010110110011000110001 \
          00001000100010000000000000000000000 10000010000010000000000000000000000 \
          00000010100000000000000000000000000 00100000000010001000100001000101110 \
          00100001000000000100001000010000100 00000000000110110010100101001001101 \
          00110010010100101110010010100110110 00000000000100110101000100001000010 \
          00000000000000000100010101000111111 00110010000010000010011111000101110 \
          00000000000111010000111101000001110 00110001000010000100001000010000110 \
          00000010000010000010001100100110001 00000000001001010010100101110110000 \
          00000000001111101010010100101010011 00000001100100101001011100100010000 \
          11111010000010000010001000100011111 00000000000111110010100101001001100 \
          00000000000111110100001000010100010 00100001000111010101011100010000100 \
          01110100011000110001010100101011011 11111000001000101010001000101010001 \
          11111000001000110001011110000101110 00101000100010100000000000000000000 \
          00000000000000000000000000000010101 00000000100011001110001100001000000 \
          00000000001110011100111000000000000 00010000100001000100010000100001000 \
          00000000000000001110000000000000000 01110000100111001000011100000000000 \
          01110010100111000000000000000000000 01100000100010000010011000000000000 \
          00000000000000000000000000000000000 00100000001100001000010000101000100 \
          00100010101111010001111101000010000 10000010010101000100010101001000001 \
          11111100011010011100101001000010000 00000000000111010001111101000001100 \
          00000000000010000100001000010000111 10001110011110111111101111001110001 \
          10100010100010100101001010101010100 00000010001110011110111000100000000 \
          11111111111111111111111111111111111 11111110111000101010110111101111011 \
          11111110111010110001101011010111111 11111110111110111001101011100111111 \
          00000000000000000000000000000011111 00100011101010100100001000000011111 \
          00100010100111001010010100000011111 00100000100011001010001100000011111 \
          00000000000100000100000100000100000 11000111000111000111000110000100000 \
          11111011110011100011000010000000000 00000000001000011000111001111011111 \
          00000000100010111101001010001000000 00000001000101001010010100010000000 \
          00000100000000000100000000000100000 00100011101111101110011100111000000 \
          00000011100111001110111110111000100 10101010101010101010101010101010101 \
          00100011111010001110001011111000100 00100011101111101110011100000000000 \
          01100100101001010110100011000110110 00000000000000000000000000000000000 \
          00000000000000000000000000000000000 00000000000000000000000000000000000 \
          00000000000000000000000000000000000 00000000000000000000000000000000000 \
          00000000000000000000000000000000000 00000000000000000000000000000000000 \
          00000000000000000000000000000000000 00000000000000000000000000000000000 \
          00000000000000000000000000000000000 00000000000000000000000000000000000)
}

function main()
{

  trap quitit HUP
  trap quitit INT
  trap quitit QUIT
  trap quitit ABRT
  trap quitit TERM

    for opt in $@
    do
        [ $opt == "--help" ] && aboutit
    done

    if [ "$1 " == " " ]
    then
      initit
    else
      loadit "$1" || initit
    fi

  stty -echo &>/dev/null
  clear
  redraw

    while :
    do

      read -s -n 1 key

        if [ "$key " == "A " -o "$key " == "k " ] #UP
        then
            if((CURY > 0))
            then
              ((CURY--))
              redraw
            fi
        fi

        if [ "$key " == "B " -o "$key " == "j " ] #DOWN
        then
            if((CURY < 6))
            then
              ((CURY++))
              redraw
            fi
        fi

        if [ "$key " == "C " -o "$key " == "l " ] #RIGHT
        then
            if((CURX < 4))
            then
              ((CURX++))
              redraw
            fi
        fi

        if [ "$key " == "D " -o "$key " == "h " ] #LEFT
        then
            if((CURX > 0))
            then
              ((CURX--))
              redraw
            fi
        fi

        if [ "$key " == "b " ] #Back
        then
            if((IDX > 0))
            then
              ((IDX--))
              redraw
            fi
        fi

        if [ "$key " == "n " ] #Next
        then
            if((IDX < 255))
            then
              ((IDX++))
              redraw
            fi
        fi

        if [ "$key " == " " ] #Edit
        then
          ((CURPOS=CURY*5+CURX))
          BUF=${CFONTS[$IDX]}
          C=${BUF:$CURPOS:1}
            ((CURPOS == 0)) && BUF1="" || BUF1=${BUF:0:$CURPOS}
            ((CURPOS == 34)) && BUF2="" || BUF2=${BUF:$((CURPOS+1))}
          CFONTS[$IDX]="${BUF1}$((1-C))${BUF2}"
          redraw
        fi

        if [ "$key " == "1 " ] #Load
        then
          SRC=$(dialog --stdout --title "Load from 8xfont file..." --fselect ./ 0 0) && loadit "$SRC"
          clear
          redraw
        fi

        if [ "$key " == "2 " ] #Save
        then
          SRC=$(dialog --stdout --title "Save as 8xfont file..." --fselect ./ 0 0) && saveit "$SRC"
          clear
          redraw
        fi

        if [ "$key " == "3 " ] #Compile
        then
          SRC=$(dialog --stdout --title "Compile to 8xp file..." --fselect ./ 0 0) && compileit "$SRC"
          clear
          redraw
        fi

        if [ "$key " == "q " ] #Quit
        then
          stty echo &>/dev/null
          clear
          exit 0
        fi

    done
}


  main "$@"

